@@ -23,22 +23,20 @@ from clipping.hints import (Contour,
                             Multisegment,
                             Point,
                             Segment)
+from . import bounding_box
 from .enums import EdgeType
 from .event import ShapedEvent as Event
 from .events_queue import (EventsQueue,
                            EventsQueueKey)
 from .sweep_line import SweepLine
 from .utils import (all_equal,
-                    are_bounding_boxes_disjoint,
-                    flatten,
+                    contour_to_segments,
                     sort_pair,
-                    to_bounding_box,
                     to_first_boundary_vertex,
                     to_multipolygon_base,
                     to_multipolygon_contours,
                     to_multipolygon_x_max,
-                    to_rational_multipolygon,
-                    to_segments)
+                    to_rational_multipolygon)
 
 
 class Operation(ABC):
@@ -64,11 +62,6 @@ class Operation(ABC):
 
     __repr__ = generate_repr(__init__)
 
-    def are_operands_bounding_boxes_disjoint(self) -> bool:
-        return are_bounding_boxes_disjoint(
-                to_bounding_box(flatten(border for border, _ in self.left)),
-                to_bounding_box(flatten(border for border, _ in self.right)))
-
     @abstractmethod
     def compute(self) -> Union_[Multipolygon, Mix]:
         """
@@ -160,10 +153,10 @@ class Operation(ABC):
 
     def fill_queue(self) -> None:
         for contour in to_multipolygon_contours(self.left):
-            for segment in to_segments(contour):
+            for segment in contour_to_segments(contour):
                 self.register_segment(segment, True)
         for contour in to_multipolygon_contours(self.right):
-            for segment in to_segments(contour):
+            for segment in contour_to_segments(contour):
                 self.register_segment(segment, False)
 
     @abstractmethod
@@ -228,8 +221,15 @@ class Difference(Operation):
     __slots__ = ()
 
     def compute(self) -> Multipolygon:
-        if (not (self.left and self.right)
-                or self.are_operands_bounding_boxes_disjoint()):
+        if not (self.left and self.right):
+            return self.left
+        left_bounding_box = bounding_box.from_multipolygon(self.left)
+        if bounding_box.disjoint_with(
+                left_bounding_box, bounding_box.from_multipolygon(self.right)):
+            return self.left
+        self.right = bounding_box.to_coupled_polygons(left_bounding_box,
+                                                      self.right)
+        if not self.right:
             return self.left
         self.normalize_operands()
         return events_to_multipolygon(self.sweep())
@@ -258,8 +258,17 @@ class Intersection(Operation):
     __slots__ = ()
 
     def compute(self) -> Multipolygon:
-        if (not (self.left and self.right)
-                or self.are_operands_bounding_boxes_disjoint()):
+        if not (self.left and self.right):
+            return []
+        left_bounding_box = bounding_box.from_multipolygon(self.left)
+        right_bounding_box = bounding_box.from_multipolygon(self.right)
+        if bounding_box.disjoint_with(left_bounding_box, right_bounding_box):
+            return []
+        self.left = bounding_box.to_coupled_polygons(right_bounding_box,
+                                                     self.left)
+        self.right = bounding_box.to_coupled_polygons(left_bounding_box,
+                                                      self.right)
+        if not (self.left and self.right):
             return []
         self.normalize_operands()
         return events_to_multipolygon(self.sweep())
@@ -288,8 +297,17 @@ class CompleteIntersection(Intersection):
     __slots__ = ()
 
     def compute(self) -> Mix:
-        if (not (self.left and self.right)
-                or self.are_operands_bounding_boxes_disjoint()):
+        if not (self.left and self.right):
+            return [], [], []
+        left_bounding_box = bounding_box.from_multipolygon(self.left)
+        right_bounding_box = bounding_box.from_multipolygon(self.right)
+        if bounding_box.disjoint_with(left_bounding_box, right_bounding_box):
+            return [], [], []
+        self.left = bounding_box.to_intersecting_polygons(right_bounding_box,
+                                                          self.left)
+        self.right = bounding_box.to_intersecting_polygons(left_bounding_box,
+                                                           self.right)
+        if not (self.left and self.right):
             return [], [], []
         self.normalize_operands()
         events = sorted(self.sweep(),
@@ -326,7 +344,9 @@ class SymmetricDifference(Operation):
     def compute(self) -> Multipolygon:
         if not (self.left and self.right):
             return self.left or self.right
-        elif self.are_operands_bounding_boxes_disjoint():
+        elif bounding_box.disjoint_with(
+                bounding_box.from_multipolygon(self.left),
+                bounding_box.from_multipolygon(self.right)):
             result = self.left + self.right
             result.sort(key=to_first_boundary_vertex)
             return result
@@ -343,7 +363,9 @@ class Union(Operation):
     def compute(self) -> Multipolygon:
         if not (self.left and self.right):
             return self.left or self.right
-        elif self.are_operands_bounding_boxes_disjoint():
+        elif bounding_box.disjoint_with(
+                bounding_box.from_multipolygon(self.left),
+                bounding_box.from_multipolygon(self.right)):
             result = self.left + self.right
             result.sort(key=to_first_boundary_vertex)
             return result
